# pyenchant
#
# Copyright (C) 2004-2006, Ryan Kelly
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPsE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# In addition, as a special exception, you are
# given permission to link the code of this program with
# non-LGPL Spelling Provider libraries (eg: a MSFT Office
# spell checker backend) and distribute linked combinations including
# the two.  You must obey the GNU Lesser General Public License in all
# respects for all of the code used other than said providers.  If you modify
# this file, you may extend this exception to your version of the
# file, but you are not obligated to do so.  If you do not wish to
# do so, delete this exception statement from your version.
#
"""
    enchant:  Access to the enchant spellchecking library

    This module provides several classes for performing spell checking
    via the Enchant spellchecking library.  For more details on Enchant,
    visit the project website:

        http://www.abisource.com/enchant/

    Spellchecking is performed using 'Dict' objects, which represent
    a language dictionary.  Their use is best demonstrated by a quick
    example:

        >>> import enchant
        >>> d = enchant.Dict("en_US")   # create dictionary for US English
        >>> d.check("enchant")
        True
        >>> d.check("enchnt")
        False
        >>> d.suggest("enchnt")
        ['enchant', 'enchants', 'enchanter', 'penchant', 'incant', 'enchain', 'enchanted']

    Languages are identified by standard string tags such as "en" (English)
    and "fr" (French).  Specific language dialects can be specified by
    including an additional code - for example, "en_AU" refers to Australian
    English.  The later form is preferred as it is more widely supported.

    To check whether a dictionary exists for a given language, the function
    'dict_exists' is available.  Dictionaries may also be created using the
    function 'request_dict'.

    A finer degree of control over the dictionaries and how they are created
    can be obtained using one or more 'Broker' objects.  These objects are
    responsible for locating dictionaries for a specific language.
    
    Unicode strings are supported transparently, as they are throughout
    Python - if a unicode string is given as an argument, the result will
    be a unicode string.  Note that Enchant works in UTF-8 internally,
    so passing an ASCII string to a dictionary for a language requiring
    Unicode may result in UTF-8 strings being returned.

    Errors that occur in this module are reported by raising 'Error'.

"""

# Make version info available
__ver_major__ = 1
__ver_minor__ = 2
__ver_patch__ = 0
__ver_sub__ = ""
__version__ = "%d.%d.%d%s" % (__ver_major__,__ver_minor__,
                              __ver_patch__,__ver_sub__)

# Define Error class before imports, so it is available for import
# by enchant subpackages with circular dependencies
class Error(Exception):
    """Base exception class for the enchant module."""
    pass


import _enchant as _e
import utils
import unittest

import os

class DictNotFoundError(Error):
    """Exception raised when a requested dictionary could not be found."""
    pass

class ProviderDesc(object):
    """Simple class describing an Enchant provider.
    Each provider has the following information associated with it:

        * name:        Internal provider name (e.g. "aspell")
        * desc:        Human-readable description (e.g. "Aspell Provider")
        * file:        Location of the library containing the provider

    """

    def __init__(self,name,desc,file):
        self.name = name
        self.desc = desc
        self.file = file

    def __str__(self):
        return "<Enchant: %s>" % self.desc

    def __repr__(self):
        return str(self)

    def __eq__(self,pd):
        """Equality operator on ProviderDesc objects."""
        return (self.name == pd.name and \
                self.desc == pd.desc and \
                self.file == pd.file)
                
    def __hash__(self):
        """Hash operator on ProviderDesc objects."""
        return hash(self.name + self.desc + self.file)


class _EnchantObject(object):
    """Base class for enchant objects.
    
    This class implements some general functionality for interfacing with
    the '_enchant' C-library in a consistent way.  All public objects
    from the 'enchant' module are subclasses of this class.
    
    All enchant objects have an attribute '_this' which contains the
    pointer to the underlying C-library object.  The method '_check_this'
    can be called to ensure that this point is not None, raising an
    exception if it is.
    """
    
    def __init__(self):
        """_EnchantObject constructor."""
        self._this = None
        
    def _check_this(self,msg=None):
         """Check that self._this is set to a pointer, rather than None."""
         if msg is None:
            msg = "%s unusable: the underlying C-library object has been freed."
            msg = msg % (self.__class__.__name__,)
         if self._this is None:
            raise Error(msg)
             
    def _raise_error(self,default="Unspecified Error",eclass=Error):
         """Raise an exception based on available error messages.
         This method causes an Error to be raised.  Subclasses should
         override it to retreive an error indication from the underlying
         API if possible.  If such a message cannot be retreived, the
         argument value <default> is used.  The class of the exception
         can be specified using the argument <eclass>
         """
         raise eclass(default)



class Broker(_EnchantObject):
    """Broker object for the Enchant spellchecker.

    Broker objects are responsible for locating and managing dictionaries.
    Unless custom functionality is required, there is no need to use Broker
    objects directly. The 'enchant' module provides a default broker object
    so that 'Dict' objects can be created directly.

    The most important methods of this class include:

        * dict_exists:   check existence of a specific language dictionary
        * request_dict:  obtain a dictionary for specific language
        * set_ordering:  specify which dictionaries to try for for a
                         given language.

    """

    # Because of the way the underlying enchant library caches dictionary
    # objects, it's dangerous to free dictionaries when more than one has
    # been created for the same language.  To work around this transparently,
    # keep track of how many Dicts have been created for each language.
    # Only call the underlying dict_free when this reaches zero.  This is
    # done in the __live_dicts attribute.

    def __init__(self):
        """Broker object constructor.
        
        This method is the constructor for the 'Broker' object.  No
        arguments are required.
        """
        _EnchantObject.__init__(self)
        self._this = _e.enchant_broker_init()
        if not self._this:
            raise Error("Could not initialise an enchant broker.")
        self.__live_dicts = {}

    def __del__(self):
        """Broker object destructor."""
        # Calling _free() might fail if python is shutting down
        try:
            self._free()
        except AttributeError:
            pass
            
    def _raise_error(self,default="Unspecified Error",eclass=Error):
        """Overrides _EnchantObject._raise_error to check broker errors."""
        err = _e.enchant_broker_get_error(self._this)
        if err == "" or err is None:
            raise eclass(default)
        raise eclass(err)

    def _free(self):
        """Free system resource associated with a Broker object.
        
        This method can be called to free the underlying system resources
        associated with a Broker object.  It is called automatically when
        the object is garbage collected.  If called explicitly, the
        Broker and any associated Dict objects must no longer be used.
        """
        if self._this is not None:
            _e.enchant_broker_free(self._this)
            self._this = None
            self.__live_dicts.clear()
            
    def __inc_live_dicts(self,tag):
        """Increment the count of live Dict objects for the given tag.
        Returns the new count of live Dicts.
        """
        try:
            self.__live_dicts[tag] += 1
        except KeyError:
            self.__live_dicts[tag] = 1
        assert(self.__live_dicts[tag] > 0)
        return self.__live_dicts[tag]

    def __dec_live_dicts(self,tag):
        """Decrement the count of live Dict objects for the given tag.
        Returns the new count of live Dicts.
        """
        try:
            self.__live_dicts[tag] -= 1
        except KeyError:
            self.__live_dicts[tag] = 0
        assert(self.__live_dicts[tag] >= 0)
        return self.__live_dicts[tag]
        
    def request_dict(self,tag=None):
        """Request a Dict object for the language specified by <tag>.
        
        This method constructs and returns a Dict object for the
        requested language.  'tag' should be a string of the appropriate
        form for specifying a language, such as "fr" (French) or "en_AU"
        (Australian English).  The existence of a specific language can
        be tested using the 'dict_exists' method.
        
        If <tag> is not given or is None, an attempt is made to determine
        the current language in use.  If this cannot be determined, Error
        is raised.
        
        NOTE:  this method is functionally equivalent to calling the Dict()
               constructor and passing in the <broker> argument.
               
        """
        return Dict(tag,self)

    def _request_dict_data(self,tag):
        """Request raw C-object data for a dictionary.
        This method call passes on the call to the C library, and does
        some internal bookkeeping.
        """
        self._check_this()
        new_dict = _e.enchant_broker_request_dict(self._this,tag)
        if new_dict is None:
            eStr = "Dictionary for language '%s' could not be found"
            self._raise_error(eStr % (tag,),DictNotFoundError)
        # Determine normalized tag, for live count
        key = self.__describe_dict(new_dict)[0]
        self.__inc_live_dicts(key)
        return new_dict

    def request_pwl_dict(self,pwl):
        """Request a Dict object for a personal word list.
        
        This method behaves as 'request_dict' but rather than returning
        a dictionary for a specific language, it returns a dictionary
        referencing a personal word list.  A personal word list is a file
        of custom dictionary entries, one word per line.
        """
        self._check_this()
        new_dict = _e.enchant_broker_request_pwl_dict(self._this,pwl)
        if new_dict is None:
            eStr = "Personal Word List file '%s' could not be loaded"
            self._raise_error(eStr % (pwl,))
        # Find normalized filename, use as key
        key = self.__describe_dict(new_dict)[3]
        self.__inc_live_dicts(key)
        d = Dict(False)
        d._switch_this(new_dict,self)
        return d

    def _free_dict(self,dict):
        """Free memory associated with a dictionary.
        
        This method frees system resources associated with a Dict object.
        It is equivalent to calling the object's 'free' method.  Once this
        method has been called on a dictionary, it must not be used again.
        """
        self._check_this()
        # Lookup key differs if it's a PWL or not
        if dict.tag.lower() == "personal wordlist":
            key = dict.provider.file
        else:
            key = dict.tag
        if self.__dec_live_dicts(key) == 0:
            _e.enchant_broker_free_dict(self._this,dict._this)
        dict._this = None
        dict._broker = None

    def dict_exists(self,tag):
        """Check availability of a dictionary.
        
        This method checks whether there is a dictionary available for
        the language specified by 'tag'.  It returns True if a dictionary
        is available, and False otherwise.
        """
        self._check_this()
        val = _e.enchant_broker_dict_exists(self._this,tag)
        return bool(val)

    def set_ordering(self,tag,ordering):
        """Set dictionary preferences for a language.
        
        The Enchant library supports the use of multiple dictionary programs
        and multiple languages.  This method specifies which dictionaries
        the broker should prefer when dealing with a given language.  'tag'
        must be an appropriate language specification and 'ordering' is a
        string listing the dictionaries in order of preference.  For example
        a valid ordering might be "aspell,myspell,ispell".
        The value of 'tag' can also be set to "*" to set a default ordering
        for all languages for which one has not been set explicitly.
        """
        self._check_this()
        if type(ordering) == unicode:
            ordering = ordering.encode("utf-8")
        _e.enchant_broker_set_ordering(self._this,tag,ordering)

    def describe(self):
        """Return list of provider descriptions.
        
        This method returns a list of descriptions of each of the
        dictionary providers available.  Each entry in the list is a 
        ProviderDesc object.
        """
        self._check_this()
        self.__describe_result = []
        _e.enchant_broker_describe_py(self._this,self.__describe_callback)
        return [ ProviderDesc(*r) for r in self.__describe_result]

    def __describe_callback(self,name,desc,file):
        """Collector callback for dictionary description.
        
        This method is used as a callback into the _enchant function
        'enchant_broker_describe_py'.  It collects the given arguments in
        a tuple and appends them to the list '__describe_result'.
        """
        name = name.decode("utf-8")
        desc = desc.decode("utf-8")
        file = file.decode("utf-8")
        self.__describe_result.append((name,desc,file))
        
    def list_dicts(self):
        """Return list of available dictionaries.
        
        This method returns a list of dictionaries available to the
        broker.  Each entry in the list is a two-tuple of the form:
            
            (tag,provider)
        
        where <tag> is the language lag for the dictionary and
        <provider> is a ProviderDesc object describing the provider
        through which that dictionary can be obtained.
        """
        self._check_this()
        self.__list_dicts_result = []
        _e.enchant_broker_list_dicts_py(self._this,self.__list_dicts_callback)
        return [ (r[0],ProviderDesc(*r[1])) for r in self.__list_dicts_result]
    
    def __list_dicts_callback(self,tag,name,desc,file):
        """Collector callback for listing dictionaries.
        
        This method is used as a callback into the _enchant function
        'enchant_broker_list_dicts_py'.  It collects the given arguments into
        an appropriate tuple and appends them to '__list_dicts_result'.
        """
        name = name.decode("utf-8")
        desc = desc.decode("utf-8")
        file = file.decode("utf-8")
        self.__list_dicts_result.append((tag,(name,desc,file)))
 
    def list_languages(self):
        """List languages for which dictionaries are available.
        
        This function returns a list of language tags for which a
        dictionary is available.
        """
        langs = []
        for (tag,prov) in self.list_dicts():
            if tag not in langs:
                langs.append(tag)
        return langs
        
    def __describe_dict(self,dict_data):
        """Get the description tuple for a dict data object.
        <dict_data> must be a C-library pointer to an enchant dictionary.
        The return value is a tuple of the form:
                (<tag>,<name>,<desc>,<file>)
        """
        # Define local callback function
        cb_result = []
        def cb_func(tag,name,desc,file):
            name = name.decode("utf-8")
            desc = desc.decode("utf-8")
            file = file.decode("utf-8")
            cb_result.append((tag,name,desc,file))
        # Actual call describer function
        _e.enchant_dict_describe_py(dict_data,cb_func)
        return cb_result[0]
        

class Dict(_EnchantObject):
    """Dictionary object for the Enchant spellchecker.

    Dictionary objects are responsible for checking the spelling of words
    and suggesting possible corrections.  Each dictionary is owned by a
    Broker object, but unless a new Broker has explicitly been created
    then this will be the 'enchant' module default Broker and is of little
    interest.

    The important methods of this class include:

        * check():              check whether a word id spelled correctly
        * suggest():            suggest correct spellings for a word
        * add_to_session():     add a word to the current spellcheck session
        * add_to_pwl():         add a word to the personal dictionary
        * store_replacement():  indicate a replacement for a given word

    Information about the dictionary is available using the following
    attributes:

        * tag:        the language tag of the dictionary
        * provider:   a ProviderDesc object for the dictionary provider
    
    """

    def __init__(self,tag=None,broker=None):
        """Dict object constructor.
        
        A dictionary belongs to a specific language, identified by the
        string <tag>.  If the tag is not given or is None, an attempt to
        determine the language currently in use is made using the 'locale'
        module.  If the current language cannot be determined, Error is raised.

        If <tag> is instead given the value of False, a 'dead' Dict object
        is created without any reference to a language.  This is typically
        only useful within PyEnchant itself.  Any other non-string value
        for <tag> raises Error.
        
        Each dictionary must also have an associated Broker object which
        obtains the dictionary information from the underlying system. This
        may be specified using <broker>.  If not given, the default broker
        is used.
        """
        # Superclass initialisation
        _EnchantObject.__init__(self)
        # Initialise object attributes to None
        self._broker = None
        self.tag = None
        self.provider = None
        # Create dead object if False was given
        if tag is False:
            self._this = None
        else:
            if tag is None:
                tag = utils.get_default_language()
                if tag is None:
                    err = "No tag specified and default language could not "
                    err = err + "be determined."
                    raise Error(err)
            # Use module-level broker if none given
            if broker is None:
                broker = _broker
            # Use the broker to get C-library pointer data
            self._switch_this(broker._request_dict_data(tag),broker)

    def __del__(self):
        """Dict object desstructor."""
        # Calling free() might fail if python is shutting down
        try:
            self._free()
        except AttributeError:
            pass
            
    def _switch_this(self,this,broker):
        """Switch the underlying C-library pointer for this object.
        
        As all useful state for a Dict is stored by the underlying C-library
        pointer, it is very convenient to allow this to be switched at
        run-time.  Pass a new dict data object into this method to affect
        the necessary changes.  The creating Broker object (at the Python
        level) must also be provided.
                
        This should *never* *ever* be used by application code.  It's
        a convenience for developers only, replacing the clunkier <data>
        parameter to __init__ from earlier versions.
        """
        # Free old dict data
        Dict._free(self)
        # Hook in the new stuff
        self._this = this
        self._broker = broker
        # Update object properties
        desc = self.__describe(check_this=False)
        self.tag = desc[0]
        self.provider = ProviderDesc(*desc[1:])
            
    def _check_this(self,msg=None):
        """Extend _EnchantObject._check_this() to check Broker validity.
        
        It is possible for the managing Broker object to be freed without
        freeing the Dict.  Thus validity checking must take into account
        self._broker._this as well as self._this.
        """
        if self._broker is None or self._broker._this is None:
            self._this = None
        _EnchantObject._check_this(self,msg)

    def _raise_error(self,default="Unspecified Error",eclass=Error):
        """Overrides _EnchantObject._raise_error to check dict errors."""
        err = _e.enchant_dict_get_error(self._this)
        if err == "" or err is None:
            raise eclass(default)
        raise eclass(err)

    def _free(self):
        """Free the system resources associated with a Dict object.
        
        This method frees underlying system resources for a Dict object.
        Once it has been called, the Dict object must no longer be used.
        It is called automatically when the object is garbage collected.
        """
        if self._broker is not None:
            self._broker._free_dict(self)

    def check(self,word):
        """Check spelling of a word.
        
        This method takes a word in the dictionary language and returns
        True if it is correctly spelled, and false otherwise.
        """
        self._check_this()
        if type(word) == unicode:
            inWord = word.encode("utf-8")
        else:
            inWord = word
        val = _e.enchant_dict_check(self._this,inWord,len(inWord))
        if val == 0:
            return True
        if val > 0:
            return False
        self._raise_error()

    def suggest(self,word):
        """Suggest possible spellings for a word.
        
        This method tries to guess the correct spelling for a given
        word, returning the possibilities in a list.
        """
        self._check_this()
        if type(word) == unicode:
            inWord = word.encode("utf-8")
        else:
            inWord = word
        suggs = _e.enchant_dict_suggest_py(self._this,inWord,len(inWord))
        if type(word) == unicode:
            uSuggs = [w.decode("utf-8") for w in suggs]
            return uSuggs
        return suggs

    def add_to_pwl(self,word):
        """Add a word to the user's personal word list."""
        self._check_this()
        if type(word) == unicode:
            inWord = word.encode("utf-8")
        else:
            inWord = word
        _e.enchant_dict_add_to_pwl(self._this,inWord,len(inWord))

    def add_to_session(self,word):
        """Add a word to the session list."""
        self._check_this()
        if type(word) == unicode:
            inWord = word.encode("utf-8")
        else:
            inWord = word
        _e.enchant_dict_add_to_session(self._this,inWord,len(inWord))

    def is_in_session(self,word):
        """Check whether a word is in the session list."""
        self._check_this()
        if type(word) == unicode:
            inWord = word.encode("utf-8")
        else:
            inWord = word
        return _e.enchant_dict_is_in_session(self._this,inWord,len(inWord))

    def store_replacement(self,mis,cor):
        """Store a replacement spelling for a miss-spelled word.
        
        This method makes a suggestion to the spellchecking engine that the 
        miss-spelled word <mis> is in fact correctly spelled as <cor>.  Such
        a suggestion will typically mean that <cor> appears early in the
        list of suggested spellings offered for later instances of <mis>.
        """
        self._check_this()
        if type(mis) == unicode:
            inMis = mis.encode("utf-8")
        else:
            inMis = mis
        if type(cor) == unicode:
            inCor = cor.encode("utf-8")
        else:
            inCor = cor
        _e.enchant_dict_store_replacement(self._this,inMis,len(inMis),inCor,len(inCor))

    def __describe(self,check_this=True):
        """Return a tuple describing the dictionary.
        
        This method returns a four-element tuple describing the underlying
        spellchecker system providing the dictionary.  It will contain the
        following strings:
            * language tag
            * name of dictionary provider
            * description of dictionary provider
            * dictionary file
        Direct use of this method is not recommended - instead, access this
        information through the 'tag' and 'provider' attributes.
        """
        if check_this:
            self._check_this()
        _e.enchant_dict_describe_py(self._this,self.__describe_callback)
        return self.__describe_result

    def __describe_callback(self,tag,name,desc,file):
        """Collector callback for dictionary description.
        
        This method is used as a callback into the _enchant function
        'enchant_dict_describe_py'.  It collects the given arguments in
        a tuple and stores them in the attribute '__describe_result'.
        """
        name = name.decode("utf-8")
        desc = desc.decode("utf-8")
        file = file.decode("utf-8")
        self.__describe_result = (tag,name,desc,file)



class DictWithPWL(Dict):
    """Dictionary with managed personal word list.
    
    This class behaves as the standard Dict class, but also manages a
    personal word list stored in a seperate file.  The file must be
    specified at creation time by the 'pwl' argument to the constructor.
    Words added to the dictionary using "add_to_personal" are automatically
    appended to the pwl file.
    
    The Dict object managing the PWL is available as the 'pwl' attribute.
    
    To create a DictWithPWL from the user's default language, use None
    as the 'tag' argument.
    """
    
    def __init__(self,tag,pwl,broker=None):
        """DictWithPWL constructor.

        The argument 'pwl' must be supplied, naming a file containing
	    the personal word list.  If this file does not exist, it is
        	created with default permissions.
        """
        if pwl is None:
            raise Error("DictWithPWL must be given a PWL file.")
        if not os.path.exists(pwl):
            f = file(pwl,"wt")
	    f.close()
	    del f
        Dict.__init__(self,tag,broker)
        self.pwl = self._broker.request_pwl_dict(pwl)
     
    def _check_this(self,msg=None):
       """Extend Dict._check_this() to check PWL validity."""
       if self.pwl is None:
           self._free()
       Dict._check_this(self,msg)
       self.pwl._check_this(msg)

    def _free(self):
        """Extend Dict._free() to free the PWL as well."""
        if self.pwl is not None:
            self.pwl._free()
            self.pwl = None
        Dict._free(self)
        
    def check(self,word):
        """Check spelling of a word.
        
        This method takes a word in the dictionary language and returns
        True if it is correctly spelled, and false otherwise.  It checks
        both the dictionary and the personal wordlist.
        """
        if Dict.check(self,word):
            return True
        if self.pwl.check(word):
            return True
        return False

    def add_to_pwl(self,word):
        """Add a word to the associated personal word list.
        
        This method adds the given word to the personal word list, and
        automatically saves the list to disk.
        """
        self._check_this()
        self.pwl.add_to_pwl(word)

##  Create a module-level default broker object, and make its important
##  methods available at the module level.
_broker = Broker()
request_dict = _broker.request_dict
request_pwl_dict = _broker.request_pwl_dict
dict_exists = _broker.dict_exists
list_dicts = _broker.list_dicts
list_languages = _broker.list_languages


##  Define unittest TestCases for the functionality provided in this module

class TestBroker(unittest.TestCase):
    """Test cases for the proper functioning of Broker objects.
    These tests assume that there is at least one working provider
    with a dictionary for the "en_US" language.
    """
    
    def setUp(self):
        self.broker = Broker()
    
    def tearDown(self):
        del self.broker

    def test_HasENUS(self):
        """Test that the en_US language is available."""
        self.assert_(self.broker.dict_exists("en_US"))
    
    def test_LangsAreAvail(self):
        """Test whether all advertised languages are in fact available."""
        for lang in self.broker.list_languages():
            self.assert_(self.broker.dict_exists(lang))
            
    def test_ProvsAreAvail(self):
        """Test whether all advertised providers are in fact available."""
        for (lang,prov) in self.broker.list_dicts():
            self.assert_(self.broker.dict_exists(lang))
            self.assert_(prov in self.broker.describe())
    
    def test_ProvOrdering(self):
        """Test that provider ordering works correctly."""
        langs = {}
        provs = []
        # Find the providers for each language, and a list of all providers
        for (tag,prov) in self.broker.list_dicts():
            if langs.has_key(tag):
                langs[tag].append(prov)
            else:
                langs[tag] = [prov]
            if prov not in provs:
                provs.append(prov)
        # Check availability using a single entry in ordering
        for tag in langs:
            for prov in langs[tag]:
                b2 = Broker()
                b2.set_ordering(tag,prov.name)
                d = b2.request_dict(tag)
                self.assertEqual(d.provider,prov)
                del d
                del b2
        # Place providers that dont have the language in the ordering
        for tag in langs:
            for prov in langs[tag]:
                order = prov.name
                for prov2 in provs:
                    if prov2 not in langs[tag]:
                        order = prov2.name + "," + order
                b2 = Broker()
                b2.set_ordering(tag,order)
                d = b2.request_dict(tag)
                self.assertEqual(d.provider,prov)
                del d
                del b2

    def test_LiveDicts(self):
        """Test proper functioning of live dicts count."""
        self.failIf(self.broker._Broker__live_dicts.has_key("en_US"))
        d1 = self.broker.request_dict("en_US")
        self.assertEqual(self.broker._Broker__live_dicts["en_US"],1)
        d2 = self.broker.request_dict("en_US")
        self.assertEqual(self.broker._Broker__live_dicts["en_US"],2)
        del d2
        self.assertEqual(self.broker._Broker__live_dicts["en_US"],1)
        d2 = self.broker.request_dict("en_US")
        self.assertEqual(self.broker._Broker__live_dicts["en_US"],2)
        del d1
        del d2
        self.assertEqual(self.broker._Broker__live_dicts["en_US"],0)
        
    def test_LiveDictsNorm(self):
        """Test live dicts count with normalised tag names."""
        self.failIf(self.broker._Broker__live_dicts.has_key("en_US"))
        d1 = self.broker.request_dict("en_US@fake")
        self.assert_(self.broker._Broker__live_dicts["en_US"] == 1)
        d2 = self.broker.request_dict("en_US.utf-8")
        self.assert_(self.broker._Broker__live_dicts["en_US"] == 2)
        del d1
        del d2
        self.assert_(self.broker._Broker__live_dicts["en_US"] == 0)


class TestDict(unittest.TestCase):
    """Test cases for the proper functioning of Dict objects.
    These tests assume that there is at least one working provider
    with a dictionary for the "en_US" language.
    """
        
    def setUp(self):
        self.dict = Dict("en_US")
    
    def tearDown(self):
        del self.dict

    def test_HasENUS(self):
        """Test that the en_US language is available."""
        self.assert_(dict_exists("en_US"))
    
    def test_check(self):
        """Test that check() works on some common words."""
        self.assert_(self.dict.check("hello"))
        self.assert_(self.dict.check("test"))
        self.failIf(self.dict.check("helo"))
        self.failIf(self.dict.check("testt"))
        
    def test_broker(self):
        """Test that the dict's broker is set correctly."""
        self.assert_(self.dict._broker is _broker)
    
    def test_tag(self):
        """Test that the dict's tag is set correctly."""
        self.assertEqual(self.dict.tag,"en_US")
    
    def test_suggest(self):
        """Test that suggest() gets simple suggestions right."""
        self.assert_("hello" in self.dict.suggest("helo"))

    def test_suggestHang1(self):
        """Test whether suggest() hangs on some inputs (Bug #1404196)"""
        self.assert_(len(self.dict.suggest("Thiis")) >= 0)
        self.assert_(len(self.dict.suggest("Thiiis")) >= 0)
        self.assert_(len(self.dict.suggest("Thiiiis")) >= 0)

    def test_unicode1(self):
        """Test checking/suggesting for unicode strings"""
        us1 = u"\u21496"
        self.assert_(type(us1) == unicode)
        self.failIf(self.dict.check(us1))
        for s in self.dict.suggest(us1):
            self.assert_(type(s) == unicode)
            
    def test_session(self):
        """Test that adding words to the session works as required."""
        self.failIf(self.dict.check("Lozz"))
        self.failIf(self.dict.is_in_session("Lozz"))
        self.dict.add_to_session("Lozz")
        self.assert_(self.dict.is_in_session("Lozz"))
        self.assert_(self.dict.check("Lozz"))
            # Check the behavior of default language
    
    def test_DefaultLang(self):
        """Test behavior of default language selection."""
        defLang = utils.get_default_language()
        if defLang is None:
            # If no default language, shouldnt work
            self.assertRaises(Error,Dict)
        else:
            # If there is a default language, should use it
            # Of course, no need for the dict to actually exist
            try:
                d = Dict()
                self.assertEqual(d.tag,defLang)
            except DictNotFoundError:
                pass


class TestPWL(unittest.TestCase):
    """Test cases for the proper functioning of PWLs and DictWithPWL objects.
    These tests assume that there is at least one working provider
    with a dictionary for the "en_US" language.
    """    
    
    def setUp(self):
        import tempfile
        try:
            self.pwlFileNm = tempfile.mkstemp()[1]
        except (NameError,AttributeError):
            self.pwlFileNm = tempfile.mktemp()
    
    def tearDown(self):
        try:
            os.remove(self.pwlFileNm)
        except:
            pass
    
    def setPWLContents(self,contents):
        """Set the contents of the PWL file."""
        pwlFile = file(self.pwlFileNm,"w")
        for ln in contents:
            pwlFile.write(ln)
            pwlFile.write("\n")
        pwlFile.close()
        
    def getPWLContents(self):
        """Retreive the contents of the PWL file."""
        pwlFile = file(self.pwlFileNm,"r")
        contents = pwlFile.readlines()
        pwlFile.close()
        return contents
    
    def test_check(self):
        """Test that basic checking words on PWLs."""
        self.setPWLContents(["Sazz","Lozz"])
        d = request_pwl_dict(self.pwlFileNm)
        self.assert_(d.check("Sazz"))
        self.assert_(d.check("Lozz"))
        self.failIf(d.check("hello"))
    
    def test_add(self):
        """Test that adding words to a PWL works correctly."""
        d = request_pwl_dict(self.pwlFileNm)
        self.failIf(d.check("Flagen"))
        d.add_to_pwl("Flagen")
        self.assert_(d.check("Flagen"))
        self.assert_("Flagen\n" in self.getPWLContents())
        
    def test_suggestions(self):
        """Test getting suggestions from a PWL."""
        self.setPWLContents(["Sazz","Lozz"])
        d = request_pwl_dict(self.pwlFileNm)
        self.assert_("sazz" in d.suggest("Saz"))
        self.assert_("lozz" in d.suggest("Saz"))
        d.add_to_pwl("Flagen")
        self.assert_("flagen" in d.suggest("Flags"))
        self.failIf("sazz" in d.suggest("Flags"))
    
    def test_DWPWL(self):
        """Test functionality of DictWithPWL."""
        self.setPWLContents(["Sazz","Lozz"])
        d = DictWithPWL("en_US",self.pwlFileNm)
        self.assert_(d.check("Sazz"))
        self.assert_(d.check("Lozz"))
        self.assert_(d.check("hello"))
        self.failIf(d.check("helo"))
        self.failIf(d.check("Flagen"))
        d.add_to_pwl("Flagen")
        self.assert_(d.check("Flagen"))
        self.assert_("Flagen\n" in self.getPWLContents())
    

def testsuite():
    from enchant.checker import TestChecker
    from enchant.tokenize import TestGetTokenizer, TestFilters
    from enchant.tokenize.en import TestTokenizeEN
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestBroker))
    suite.addTest(unittest.makeSuite(TestDict))
    suite.addTest(unittest.makeSuite(TestPWL))
    suite.addTest(unittest.makeSuite(TestChecker))
    suite.addTest(unittest.makeSuite(TestGetTokenizer))
    suite.addTest(unittest.makeSuite(TestTokenizeEN))
    suite.addTest(unittest.makeSuite(TestFilters))
    return suite

# Run regression tests when called from comand-line
if __name__ == "__main__":
    UnitTest.TextTestRunner().run(testsuite())





# pyenchant
#
# Copyright (C) 2004-2005, Ryan Kelly
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# In addition, as a special exception, you are
# given permission to link the code of this program with
# non-LGPL Spelling Provider libraries (eg: a MSFT Office
# spell checker backend) and distribute linked combinations including
# the two.  You must obey the GNU Lesser General Public License in all
# respects for all of the code used other than said providers.  If you modify
# this file, you may extend this exception to your version of the
# file, but you are not obligated to do so.  If you do not wish to
# do so, delete this exception statement from your version.
#
#
# Portions of this code were automatically generated using wxGlade:
#
#         http://wxglade.sourceforge.net/
#
"""

    enchant.checker.wxSpellCheckerDialog: wxPython spellchecker interface
    
    This module provides the class wxSpellCheckerDialog, which provides
    a wxPython dialog that can be used as an interface to a spell checking
    session.  Currently it is intended as a proof-of-concept and demonstration
    class, but it should be suitable for general-purpose use in a program.
    
    The class must be given an enchant.checker.SpellChecker object with
    which to operate.  It can (in theory...) be used in modal and non-modal
    modes.  Use Show() when operating on an array of characters as it will
    modify the array in place, meaning other work can be done at the same
    time.  Use ShowModal() when operating on a static string.

"""

import wx

class wxSpellCheckerDialog(wx.Dialog):
    """Simple spellcheck dialog for wxPython
    
    This class implements a simple spellcheck interface for wxPython,
    in the form of a dialog.  It's intended mainly of an example of
    how to do this, although it should be useful for applications that
    just need a simple graphical spellchecker.
    
    The GUI code was created using wxGlade and the spellchecking related
    code lives in the following methods:
        
        * __do_init
        * The event callbacks _On<Something>
        * SetSpellChecker/GetSpellChecker
        * _Advance
    
    To use, a SpellChecker instance must be created and passed to the
    dialog before it is shown:
        
        >>> dlg = wxSpellCheckerDialog(None,-1,"")
        >>> chkr = SpellChecker("en_AU",text)
        >>> dlg.SetSpellChecker(chkr)
        >>> dlg.Show()
    
    This is most useful when the text to be checked is in the form of
    a character array, as it will be modified in place as the user
    interacts with the dialog.  For checking strings, the final result
    will need to be obtained from the SpellChecker object:
        
        >>> dlg = wxSpellCheckerDialog(None,-1,"")
        >>> chkr = SpellChecker("en_AU",text)
        >>> dlg.SetSpellChecker(chkr)
        >>> dlg.ShowModal()
        >>> text = dlg.GetSpellChecker().get_text()
    
    Currently the checker must deal with strings of the same type as
    returned by wxPython - unicode or normal string depending on the
    underlying syste.  This needs to be fixed, somehow...
    
    """
    
    def __init__(self, *args, **kwds):
        # begin wxGlade: wxSpellCheckerDialog.__init__
        kwds["style"] = wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER|wx.THICK_FRAME
        wx.Dialog.__init__(self, *args, **kwds)
        self.error_label = wx.StaticText(self, -1, "Unrecognised Word:")
        self.error_text = wx.TextCtrl(self, -1, "", style=wx.TE_MULTILINE|wx.TE_READONLY|wx.TE_RICH)
        self.replace_label = wx.StaticText(self, -1, "Replace with:")
        self.replace_text = wx.TextCtrl(self, -1, "", style=wx.TE_PROCESS_ENTER)
        #self.replace_list = wx.ListBox(self, -1, choices=["Option 1", "Option 2", "Option 3"], style=wx.LB_SINGLE)
        self.replace_list = wx.ListBox(self, -1, style=wx.LB_SINGLE)
        self.btn_ignore = wx.Button(self, -1, "Ignore")
        self.btn_ignoreall = wx.Button(self, -1, "Ignore All")
        self.btn_replace = wx.Button(self, -1, "Replace")
        self.btn_replaceall = wx.Button(self, -1, "Replace All")
        self.btn_add = wx.Button(self, -1, "Add")
        self.btn_done = wx.Button(self, -1, "Done")

        self.__set_properties()
        self.__do_layout()
        
        self.__do_init()
        self.__do_events()
        # end wxGlade

    def __set_properties(self):
        # begin wxGlade: wxSpellCheckerDialog.__set_properties
        self.SetTitle("Checking Spelling...")
        #self.replace_list.SetSelection(0)
        # end wxGlade

    def __do_layout(self):
        # begin wxGlade: wxSpellCheckerDialog.__do_layout
        sizer_5 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_6 = wx.BoxSizer(wx.VERTICAL)
        sizer_7_copy_4 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_7_copy_3 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_7_copy_2 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_7_copy_1 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_7_copy = wx.BoxSizer(wx.HORIZONTAL)
        sizer_7 = wx.BoxSizer(wx.HORIZONTAL)
        sizer_1 = wx.BoxSizer(wx.VERTICAL)
        sizer_2_copy = wx.BoxSizer(wx.HORIZONTAL)
        sizer_1.Add(self.error_label, 0, wx.LEFT|wx.TOP, 5)
        sizer_1.Add(self.error_text, 1, wx.ALL|wx.EXPAND, 5)
        sizer_2_copy.Add(self.replace_label, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_2_copy.Add(self.replace_text, 1, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5)
        sizer_1.Add(sizer_2_copy, 0, wx.EXPAND, 0)
        sizer_1.Add(self.replace_list, 2, wx.ALL|wx.EXPAND, 5)
        sizer_5.Add(sizer_1, 1, wx.EXPAND, 0)
        sizer_7.Add(self.btn_ignore, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_6.Add(sizer_7, 1, wx.EXPAND, 0)
        sizer_7_copy.Add(self.btn_ignoreall, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_6.Add(sizer_7_copy, 1, wx.EXPAND, 0)
        sizer_7_copy_1.Add(self.btn_replace, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_6.Add(sizer_7_copy_1, 1, wx.EXPAND, 0)
        sizer_7_copy_2.Add(self.btn_replaceall, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_6.Add(sizer_7_copy_2, 1, wx.EXPAND, 0)
        sizer_7_copy_3.Add(self.btn_add, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_6.Add(sizer_7_copy_3, 1, wx.EXPAND, 0)
        sizer_7_copy_4.Add(self.btn_done, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_6.Add(sizer_7_copy_4, 1, wx.EXPAND, 0)
        sizer_5.Add(sizer_6, 0, wx.ALL|wx.EXPAND, 5)
        self.SetAutoLayout(True)
        self.SetSizer(sizer_5)
        sizer_5.Fit(self)
        sizer_5.SetSizeHints(self)
        self.Layout()
        # end wxGlade
    
    def __do_init(self):
        """Initialise internal state for the dialog.
        Initially, no checker is available and buttons are disabled.
        """
        self._numContext = 40
        self._checker = None
        self._buttonsEnabled = True
        self._DisableButtons()
        # Remove placeholder options from replace_list
        self.replace_list.Clear()
        
    def __do_events(self):
        """Connect events for the dialog."""
        wx.EVT_BUTTON(self,self.btn_ignore.GetId(),self._OnIgnore)
        wx.EVT_BUTTON(self,self.btn_ignoreall.GetId(),self._OnIgnoreAll)
        wx.EVT_BUTTON(self,self.btn_replace.GetId(),self._OnReplace)
        wx.EVT_BUTTON(self,self.btn_replaceall.GetId(),self._OnReplaceAll)
        wx.EVT_BUTTON(self,self.btn_add.GetId(),self._OnAdd)
        wx.EVT_BUTTON(self,self.btn_done.GetId(),self._OnDone)
        wx.EVT_LISTBOX(self,self.replace_list.GetId(),self._OnReplSelect)
        wx.EVT_LISTBOX_DCLICK(self,self.replace_list.GetId(),self._OnReplace)
        
    def SetSpellChecker(self,chkr):
        """Set the spell checker, advancing to the first error."""
        self._checker = chkr
        self._Advance()
        
    def GetSpellChecker(self,chkr):
        """Get the currently in-use SpellChecker object."""
        return self._checker
    
    def _OnIgnore(self,evnt=None):
        """Callback for the "ignore" button.
        This simply advances to the next error.
        """
        self._Advance()
        
    def _OnIgnoreAll(self,evnt=None):
        """Callback for the "ignore all" button."""
        self._checker.ignore_always()
        self._Advance()
        
    def _OnReplace(self,evnt=None):
        """Callback for the "replace" button."""
        repl = self._GetRepl()
        self._checker.replace(repl)
        self._Advance()
        
    def _OnReplaceAll(self,evnt=None):
        """Callback for the "replace all" button."""
        repl = self._GetRepl()
        self._checker.replace_always(repl)
        self._Advance()
    
    def _OnDone(self,evnt=None):
        """Callback for the "close" button."""
        self.EndModal(wx.ID_OK)
    
    def _OnAdd(self,evnt=None):
        """Callback for the "add" button."""
        self._checker.add_to_personal()
    
    def _OnReplSelect(self,evnt=None):
        """Callback when a new replacement option is selected."""
        sel = self.replace_list.GetSelection()
        if sel == -1:
            return
        opt = self.replace_list.GetString(sel)
        self.replace_text.SetValue(opt)
    
    def _GetRepl(self):
        """Get the chosen replacement string."""
        repl = self.replace_text.GetValue()
        # Coercion now done automatically in SpellChecker class
        #repl = self._checker.coerce_string(repl)
        return repl

    def _Advance(self):
        """Advance to the next error.
        This method advances the SpellChecker to the next error, if
        any.  It then displays the error and some surrounding context,
        and well as listing the suggested replacements.
        """
        # Disable interaction if no checker
        if self._checker is None:
            self._DisableButtons()
            return
        # Advance to next error, disable if not available
        try:
            self._checker.next()
        except StopIteration:
            self._DisableButtons()
            self.error_text.SetValue("")
            self.replace_list.Clear()
            self.replace_text.SetValue("")
            return
        self._EnableButtons()
        # Display error context with erroneous word in red
	    # Restoring default style was misbehaving under windows, so
    	# I am forcing the rest of the text to be black
        self.error_text.SetValue("")
        self.error_text.SetDefaultStyle(wx.TextAttr(wx.BLACK))
        lContext = self._checker.leading_context(self._numContext)
        self.error_text.AppendText(lContext)
        self.error_text.SetDefaultStyle(wx.TextAttr(wx.RED))
        self.error_text.AppendText(self._checker.word)
        self.error_text.SetDefaultStyle(wx.TextAttr(wx.BLACK))
        tContext = self._checker.trailing_context(self._numContext)
        self.error_text.AppendText(tContext)
        # Display suggestions in the replacements list
        suggs = self._checker.suggest()
        self.replace_list.Clear()
        for s in suggs:
            self.replace_list.Append(s)
        if len(suggs) > 0:
            self.replace_text.SetValue(suggs[0])
        else:
            self.replace_text.SetValue("")
        
    def _EnableButtons(self):
        """Enable the checking-related buttons"""
        if self._buttonsEnabled:
            return
        self.btn_add.Enable(True)
        self.btn_ignore.Enable(True)
        self.btn_ignoreall.Enable(True)
        self.btn_replace.Enable(True)
        self.btn_replaceall.Enable(True)
        self._buttonsEnabled = True

    def _DisableButtons(self):
        """Disable the checking-related buttons"""    
        if not self._buttonsEnabled:
            return
        self.btn_add.Disable()
        self.btn_ignore.Disable()
        self.btn_ignoreall.Disable()
        self.btn_replace.Disable()
        self.btn_replaceall.Disable()
        self._buttonsEnabled = False
    
        
# end of class wxSpellCheckerDialog

def _test():
    class TestDialog(wxSpellCheckerDialog):
        def __init__(self,*args):
            wxSpellCheckerDialog.__init__(self,*args)
            wx.EVT_CLOSE(self,self.OnClose)
        def OnClose(self,evnt):
            if self._checker is not None:
                print "AFTER:", self._checker.get_text()
            self.Destroy()
    from enchant.checker import SpellChecker
    text = "This is sme text with a fw speling errors in it. Here are a fw more to tst it ut."
    print "BEFORE:", text
    app = wx.PySimpleApp()
    dlg = TestDialog(None,-1,"")
    chkr = SpellChecker("en_US",text)
    dlg.SetSpellChecker(chkr)
    dlg.Show()
    app.MainLoop()
    
if __name__ == "__main__":
    _test()




# pyenchant
#
# Copyright (C) 2004 Ryan Kelly
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# In addition, as a special exception, you are
# given permission to link the code of this program with
# non-LGPL Spelling Provider libraries (eg: a MSFT Office
# spell checker backend) and distribute linked combinations including
# the two.  You must obey the GNU Lesser General Public License in all
# respects for all of the code used other than said providers.  If you modify
# this file, you may extend this exception to your version of the
# file, but you are not obligated to do so.  If you do not wish to
# do so, delete this exception statement from your version.
#
"""

    enchant.utils:    Misc utilities for the enchant package
    
    This module provies miscellaneous utilities for use with the
    enchant spellchecking package.  Currently available functionality
    includes:
        
        * 'backported' functionality for python2.2 compatability
        * functions for dealing with locale/language settings
    
    Functionality that is temporarily available and may move or disappear
    in future releases:
        
        * useful string-handling functions (soundex, edit_dist, ...)
        * PyPWL, a pure-python PWL object with enhanced functionality
          
"""

# Get generators for python2.2
from __future__ import generators

import os

# Attempt to access local language information
try:
    import locale
except ImportError:
    locale = None

# Define basestring if it's not provided (e.g. python2.2)
try:
    basestring = basestring
except NameError:
    basestring = (str,unicode)
    
# Define enumerate() if it's not provided (e.g. python2.2)
try:
    enumerate = enumerate
except NameError:
    def enumerate(seq):
        """Iterator producing (index,value) pairs for an interable."""
        idx = 0
        for itm in seq:
            yield (idx,itm)
            idx += 1

#  Allow looking up of default language on-demand.

def get_default_language(default=None):
    """Determine the user's default language, if possible.
    
    This function uses the 'locale' module to try to determine
    the user's prefered language.  The return value is as
    follows:
        
        * if a locale is available for the LC_MESSAGES category,
          that language is used
        * if a default locale is available, that language is used
        * if the keyword argument <default> is given, it is used
        * None
        
    Note that determining the user's language is in general only
    possible if they have set the necessary environment variables
    on their system.
    """
    try:
        import locale
        tag = locale.getlocale()[0]
        if tag is None:
            tag = locale.getdefaultlocale()[0]
            if tag is None:
                raise Error("No default language available")
        return tag
    except:
        pass
    return default


def win32_data_files():
    """Get list of supporting data files, for use with setup.py
    
    This function returns a list of the supporting data files available
    to the running version of PyEnchant.  This is in the format expected
    by the data_files argument of the distutils setup function.  It's
    very useful, for example, for including the data files in an executable
    produced by py2exe.
    
    Only really tested on the win32 platform (it's the only platform for
    which we ship our own supporting data files)
    """
    dataDirs = ("share/enchant/myspell","share/enchant/ispell","lib/enchant")
    mainDir = os.path.abspath(os.path.dirname(__file__))
    dataFiles = []
    for dataDir in dataDirs:
        files = []
        fullDir = os.path.join(mainDir,os.path.normpath(dataDir))
        for fn in os.listdir(fullDir):
            fullFn = os.path.join(fullDir,fn)
            if os.path.isfile(fullFn):
                files.append(fullFn)
        dataFiles.append((dataDir,files))
    return dataFiles
     


# Useful string-handling functions

def soundex(name, len=4):
    """ soundex module conforming to Knuth's algorithm
        implementation 2000-12-24 by Gregory Jorgensen
        public domain
    """
    # digits holds the soundex values for the alphabet
    digits = '01230120022455012623010202'
    sndx = ''
    fc = ''
    # translate alpha chars in name to soundex digits
    for c in name.upper():
        if c.isalpha():
            if not fc: fc = c   # remember first letter
            d = digits[ord(c)-ord('A')]
            # duplicate consecutive soundex digits are skipped
            if not sndx or (d != sndx[-1]):
                sndx += d
    # replace first digit with first alpha character
    sndx = fc + sndx[1:]
    # remove all 0s from the soundex code
    sndx = sndx.replace('0','')
    # return soundex code padded to len characters
    return (sndx + (len * '0'))[:len]

def edit_dist(a,b):
    """Calculates the Levenshtein distance between a and b.
       From Wikipedia: http://en.wikisource.org/wiki/Levenshtein_distance
    """
    n, m = len(a), len(b)
    if n > m:
        # Make sure n <= m, to use O(min(n,m)) space
        a,b = b,a
        n,m = m,n
    current = range(n+1)
    for i in range(1,m+1):
        previous, current = current, [i]+[0]*m
        for j in range(1,n+1):
            add, delete = previous[j]+1, current[j-1]+1
            change = previous[j-1]
            if a[j-1] != b[i-1]:
                change = change + 1
            current[j] = min(add, delete, change)
    return current[n]


# Make enchant.Error available
# Done at bottom of file to avoid circular imports
from enchant import Error





# pyenchant
#
# Copyright (C) 2004-2005, Ryan Kelly
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# In addition, as a special exception, you are
# given permission to link the code of this program with
# non-LGPL Spelling Provider libraries (eg: a MSFT Office
# spell checker backend) and distribute linked combinations including
# the two.  You must obey the GNU Lesser General Public License in all
# respects for all of the code used other than said providers.  If you modify
# this file, you may extend this exception to your version of the
# file, but you are not obligated to do so.  If you do not wish to
# do so, delete this exception statement from your version.
#
"""

    enchant.tokenize:    String tokenisation functions for PyEnchant

    An important task in spellchecking is breaking up large bodies of
    text into their constituent words, each of which is then checked
    for correctness.  This package provides Python functions to split
    strings into words according to the rules of a particular language.
    
    Each tokenisation function accepts a string as its only positional
    argument, and returns an iterator which will yield tuples of the
    following form, one for each word found:
        
        (<word>,<pos>)
        
    The meanings of these fields should be clear: <word> is the word
    that was found and <pos> is the position within the text at which
    the word began (zero indexed, of course).  The function will work
    on any string-like object that supports array-slicing - in
    particular, character-array objects from the 'array' module may
    be used.
    
    The iterator also provides the attribute 'offset' which may be used
    to get/set the current position of the tokenizer inside the string
    being split.  This can be used for example if the string's contents
    have changed during the tokenisation process.
    
    To obtain an appropriate tokenisation function for the language
    identified by <tag>, use the function 'get_tokenizer(tag)'.
    
    This library is designed to be easily extendable by third-party
    authors.  To register a tokenisation function for the language
    <tag>, implement it as the function 'tokenize' within the
    module enchant.tokenize.<tag>.  The 'get_tokenizer' function
    will automatically detect it.  Note that the underscore must be
    used as the tag component seperator in this case, in order to
    form a valid python module name.
    
    Currently, a tokeniser has only been implemented for the English
    language.  Based on the author's limited experience, this should
    be at least partially suitable for other languages.

    This module also provides the Filter class and a variety of subclasses.
    These are designed to allow skipping over certain types of word
    during the spellchecking process.  For example, the following would
    produce a tokenizer for the English language that skips over URLs
    and WikiWords:
        
        tknzr = URLFilter(WikiWordsFilter(get_tokenizer("en_US")))
        
    It is then used in the same way as a tokenizer functionn:
        
        tkns = tknzr("text to be tokenized goes here")
        for (word,pos) in tkns:
            do_something(word)
        
"""

import unittest
import re

import enchant

class tokenize:
    """Base class for all tokenizer objects.
    Each tokenizer must be an interator and provide the 'offset'
    attribute as described in the documentation for this module.
    """
    def __init__(self,text):
        raise NotImplementedError()
    def next(self):
        raise NotImplementedError()
    def __iter__(self):
        return self


class Error(enchant.Error):
    """Exception subclass for the tokenize module.
    This exception is raised for errors within the enchant.tokenize
    module.
    """
    pass

def get_tokenizer(tag,fallback=False):
    """Lookup an appropriate tokenizer by language tag.
    This requires importing the function 'tokenize' from an
    appropriate module.  Modules tried are named after the
    language tag, tried in the following order:
        * the entire tag (e.g. "en_AU.py")
        * the base country code of the tag (e.g. "en.py")
    If a suitable function cannot be found, raises Error.
    If the optional argument <fallback> is True, languages for which
    a tokeniser cannot be found have the English tokeniser returned.
    It should do a 'reasonable' job in most cases.
    """
    # Ensure only '_' used as seperator
    tag = tag.replace("-","_")
    # Try the whole tag
    tokenizer = _try_tokenizer(tag)
    if tokenizer is not None:
        return tokenizer
    # Try just the base
    base = tag.split("_")[0]
    tokenizer = _try_tokenizer(base)
    if tokenizer is not None:
        return tokenizer
    if fallback:
        tokenizer = _try_tokenizer("en")
        if tokenizer is not None:
            return tokenizer
    raise Error("No tokenizer found for language '%s'" % (tag,))

def _try_tokenizer(modName):
    """Look for a tokenizer in the named module.
    Returns the function if found, None otherwise.
    """
    modBase = "enchant.tokenize."
    funcName = "tokenize"
    modName = modBase + modName
    try:
        mod = __import__(modName,globals(),{},funcName)
        return getattr(mod,funcName)
    except ImportError:
       return None


class Filter:
    """Base class for token filtering functions.
    Filters are used to skip over certain special words during a spellchecking
    session, such as URLs or WikiWords.  A filter is designed to wrap a
    tokenizer (or another filter) and skip over tokens that match its
    particular check.  So you might use them as follows:
        
        tknzr = get_tokenizer("en_US")
        tknzr = URLFilter(WikiWordsFilter(tknzr))
        
    The object in <tknzr> would then be a tokenizer function for English text
    that skips over URLs and WikiWords.
    
    Subclasses has two basic options for customising their behavior.  The
    method _match(word) may be overridden to return True for words that
    should be skipped, and false otherwise.  If more complex behavior is
    needed, the inner class TokenFilter can be overriden.
    """
    
    def __init__(self,tokenizer):
        """Filter class constructor."""
        self._tokenizer = tokenizer
    
    def __call__(self,*args,**kwds):
        tkn = self._tokenizer(*args,**kwds)
        return self.TokenFilter(tkn,self._match)
    
    def _match(self,word):
        """Filter matching method.
        If this method returns true, the given word will be skipped by
        the filter.  This should be overriden in subclasses to produce the
        desired functionality.
        """
        return False
    
    class TokenFilter:
        def __init__(self,tokenizer,match):
            self._match = match
            self._tokenizer = tokenizer
    
        def __iter__(self):
            return self
    
        def next(self):
            """Iterator protocol method for Filter objects.
            The call to next is passed on to the underlying tokenizer,
            skipping over words matching against the filter.
            """
            (word,pos) = self._tokenizer.next()
            while self._match(word):
                (word,pos) = self._tokenizer.next()
            return (word,pos)
                
        # Pass on access to offset to the tokenizer.
        def _getOffset(self):
            return self._tokenizer.offset
        def _setOffset(self,val):
            self._tokenizer.offset = val
        offset = property(_getOffset,_setOffset)


#  Pre-defined filters start here

class URLFilter(Filter):
    """Filter skipping over URLs.
    This filter skips any words matching the following regular expression:
       
           ^[a-zA-z]+:\/\/[^\s].*
        
    That is, any words that are URLs.
    """
    _pattern = re.compile(r"^[a-zA-z]+:\/\/[^\s].*")
    def _match(self,word):
        if self._pattern.match(word):
            return True
        return False

class WikiWordFilter(Filter):
    """Filter skipping over WikiWords.
    This filter skips any words matching the following regular expression:
       
           ^([A-Z]\w+[A-Z]+\w+)
        
    That is, any words that are WikiWords.
    """
    _pattern = re.compile(r"^([A-Z]\w+[A-Z]+\w+)")
    def _match(self,word):
        if self._pattern.match(word):
            return True
        return False

class EmailFilter(Filter):
    """Filter skipping over email addreses.
    This filter skips any words matching the following regular expression:
       
           ^.+@[^\.].*\.[a-z]{2,}$
        
    That is, any words that resemble email addresses.
    """
    _pattern = re.compile(r"^.+@[^\.].*\.[a-z]{2,}$")
    def _match(self,word):
        if self._pattern.match(word):
            return True
        return False

#TODO: HTMLFilter, LaTeXFilter, ...


# Test cases begin here

class TestGetTokenizer(unittest.TestCase):
    """TestCases for testing the get_tokenizer() functionality."""
    
    def test_get_tokenizer(self):
        """Simple regression test for get_tokenizer."""
        from enchant.tokenize import en
        self.assert_(get_tokenizer("en") is en.tokenize)
        self.assert_(get_tokenizer("en_AU") is en.tokenize)
        self.assert_(get_tokenizer("en_US") is en.tokenize)
        self.assertRaises(Error,get_tokenizer,"nonexistant")
        self.assert_(get_tokenizer("nonexistant",fallback=True) is en.tokenize)
    

class TestFilters(unittest.TestCase):
    """TestCases for the various Filter subclasses."""
    
    text = """this text with http://url.com and SomeLinksLike
              ftp://my.site.com.au/some/file AndOthers not:/quite.a.url
              with-an@aemail.address as well"""
    
    def setUp(self):
        from enchant.tokenize import en
        self.tokenize = en.tokenize
    
    def test_URLFilter(self):
        """Test filtering of URLs"""
        tkns = URLFilter(self.tokenize)(self.text)
        out = [t for t in tkns]
        exp = [("this",0),("text",5),("with",10),("and",30),
               ("SomeLinksLike",34),("AndOthers",93),("not:/quite.a.url",103),
               ("with-an@aemail.address",134),("as",157),("well",160)]
        self.assertEquals(out,exp)
        
    def test_WikiWordFilter(self):
        """Test filtering of WikiWords"""
        tkns = WikiWordFilter(self.tokenize)(self.text)
        out = [t for t in tkns]
        exp = [("this",0),("text",5),("with",10),("http://url.com",15),
               ("and",30), ("ftp://my.site.com.au/some/file",62),
               ("not:/quite.a.url",103),
               ("with-an@aemail.address",134),("as",157),("well",160)]
        self.assertEquals(out,exp)
        
    def test_EmailFilter(self):
        """Test filtering of email addresses"""
        tkns = EmailFilter(self.tokenize)(self.text)
        out = [t for t in tkns]
        exp = [("this",0),("text",5),("with",10),("http://url.com",15),
               ("and",30),("SomeLinksLike",34),
               ("ftp://my.site.com.au/some/file",62),("AndOthers",93),
               ("not:/quite.a.url",103),
               ("as",157),("well",160)]
        self.assertEquals(out,exp)
        
    def test_CombinedFilter(self):
        """Test several filters combined"""
        tkns=EmailFilter(URLFilter(WikiWordFilter(self.tokenize)))(self.text)
        out = [t for t in tkns]
        exp = [("this",0),("text",5),("with",10),
               ("and",30),("not:/quite.a.url",103),
               ("as",157),("well",160)]
        self.assertEquals(out,exp)
        
        


# pyenchant
#
# Copyright (C) 2004-2005, Ryan Kelly
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# In addition, as a special exception, you are
# given permission to link the code of this program with
# non-LGPL Spelling Provider libraries (eg: a MSFT Office
# spell checker backend) and distribute linked combinations including
# the two.  You must obey the GNU Lesser General Public License in all
# respects for all of the code used other than said providers.  If you modify
# this file, you may extend this exception to your version of the
# file, but you are not obligated to do so.  If you do not wish to
# do so, delete this exception statement from your version.
#
"""

    enchant.tokenize.en:    Tokeniser for the English language
    
    This module implements a PyEnchant text tokenizer for the English
    language, based on very simple rules.

"""

import unittest
import enchant.tokenize

class tokenize(enchant.tokenize.tokenize):
    """Iterator splitting text into words, reporting position.
    
    This iterator takes a text string as input, and yields tuples
    representing each distinct word found in the text.  The tuples
    take the form:
        
        (<word>,<pos>)
        
    Where <word> is the word string found and <pos> is the position
    of the start of the word within the text.
    """
    
    # Chars to remove from start/end of words
    strip_from_start = '"' + "'`({[<>]})"
    strip_from_end = '"' + "'`({[<>]}).!,?;:"
    
    def __init__(self,text):
        self._text = text
        self.offset = 0

    def next(self):
        text = self._text
        offset = self.offset
        while True:
            if offset >= len(text):
                break
            # Find start of next word
            while offset < len(text) and text[offset].isspace():
                offset += 1
            sPos = offset
            # Find end of word
            while offset < len(text) and not text[offset].isspace():
                offset += 1
            ePos = offset
            self.offset = offset
            # Strip chars from font/end of word
            while text[sPos] in self.strip_from_start:
                sPos += 1
            while text[ePos-1] in self.strip_from_end:
                    ePos -= 1
            # Return if word isnt empty
            if(sPos != ePos):
                return (text[sPos:ePos],sPos)
        raise StopIteration()


class TestTokenizeEN(unittest.TestCase):
    """TestCases for checking behavior of English tokenization."""
    
    def test_tokenize_en(self):
        """Simple regression test for english tokenization."""
        input = """This is a paragraph.  It's not very special, but it's designed
2 show how the splitter works with many-different combos
of words. Also need to "test" the (handling) of 'quoted' words."""
        output = [
                  ("This",0),("is",5),("a",8),("paragraph",10),("It's",22),
                  ("not",27),("very",31),("special",36),("but",45),("it's",49),
                  ("designed",54),("2",63), ("show",65),("how",70),("the",74),
                  ("splitter",78),("works",87),("with",93),("many-different",98),
                  ("combos",113),("of",120),("words",123),
                  ("Also",130),("need",135),
                  ("to",140),("test",144),("the",150),("handling",155),
                  ("of",165),("quoted",169),("words",177)
                 ]
        for (itmO,itmV) in zip(output,tokenize(input)):
            self.assertEqual(itmO,itmV)


    
        



# pyenchant
#
# Copyright (C) 2004-2005, Ryan Kelly
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
# In addition, as a special exception, you are
# given permission to link the code of this program with
# non-LGPL Spelling Provider libraries (eg: a MSFT Office
# spell checker backend) and distribute linked combinations including
# the two.  You must obey the GNU Lesser General Public License in all
# respects for all of the code used other than said providers.  If you modify
# this file, you may extend this exception to your version of the
# file, but you are not obligated to do so.  If you do not wish to
# do so, delete this exception statement from your version.
#
"""

    enchant.checker:  High-level spellchecking functionality
    
    This package is designed to host higer-level spellchecking functionality
    than is available in the base enchant package.  It should make writing
    applications that follow common usage idioms significantly easier.
    
    The most useful class is SpellChecker, which implements a spellchecking
    loop over a block of text.  It is capable of modifying the text in-place
    if given an array of characters to work with.
    
    This package also contains several interfaces to the SpellChecker class,
    such as a wxPython GUI dialog and a command-line interface.

"""

import array
import unittest

import enchant
from enchant.tokenize import get_tokenizer
from enchant.utils import basestring, enumerate

class SpellChecker:
    """Class implementing stateful spellchecking behavior.
        
    This class is designed to implement a spell-checking loop over
    a block of text, correcting/ignoring/replacing words as required.
    This loop is implemented using an iterator paradigm so it can be
    embedded inside other loops of control.
    
    The SpellChecker object is stateful, and the appropriate methods
    must be called to alter its state and affect the progress of
    the spell checking session.  At any point during the checking
    session, the attribute 'word' will hold the current erroneously
    spelled word under consideration.  The action to take on this word
    is determined by calling methods such as 'replace', 'replace_always'
    and 'ignore_always'.  Once this is done, calling 'next' advances
    to the next misspelled word.
    
    As a quick (and rather silly) example, the following code replaces
    each misspelled word with the string "ERROR":
        
        >>> text = "This is sme text with a fw speling errors in it."
        >>> chkr = SpellChecker("en_US",text)
        >>> for err in chkr:
        ...   err.replace("ERROR")
        ...
        >>> chkr.get_text()
        'This is ERROR text with a ERROR ERROR errors in it.'
        >>>

    Internally, the SpellChecker always works with arrays of (possibly
    unicode) character elements.  This allows the in-place modification
    of the string as it is checked, and is the closest thing Python has
    to a mutable string.  The text can be set as any of a normal string,
    unicode string, character array or unicode character array. The
    'get_text' method will return the modified array object if an
    array is used, or a new string object if a string it used.
    
    Words input to the SpellChecker may be either plain strings or
    unicode objects.  They will be converted to the same type as the
    text being checked, using python's default encoding/decoding
    settings.
    
    If using an array of characters with this object and the
    array is modified outside of the spellchecking loop, use the
    'set_offet' method to reposition the internal loop pointer
    to make sure it doesnt skip any words.
    
    """
    
    def __init__(self,lang,text=None,tokenize=None,filters=()):
        """Constructor for the SpellChecker class.
        SpellChecker objects can be created in two ways, depending on
        the nature of the first argument.  If it is a string, it
        specifies a language tag from which a dictionary is created.
        Otherwise, it must be an enchant Dict object to be used.
        
        Optional keyword arguments are:
            
            * text:  to set the text to be checked at creation time
            * tokenize:  a custom tokenization function to use
            * filters:  a list of filters to apply during tokenization
            
        If <tokenize> is not given and the first argument is a Dict,
        its 'tag' attribute must be a language tag so that a tokenization
        function can be created automatically.  In particular this means
        that a PWL dictionary cannot be used without specifying a
        tokenization function.
        """
        if isinstance(lang,basestring):
            dict = enchant.Dict(lang)
        else:
            dict = lang
            lang = dict.tag
        self.lang = lang
        self.dict = dict
        if tokenize is None:
            tokenize = get_tokenizer(lang,fallback=True)
        for f in filters:
            tokenize = f(tokenize)
        self._tokenize = tokenize
        
        self.word = None
        self.wordpos = None
        self._tokens = ()
        self._ignore_words = []
        self._replace_words = {}
        self._text = array.array('u')
        self._use_tostring = False
        
        if text is not None:
            self.set_text(text)

    def __iter__(self):
        """Each SpellChecker object is its own iterator"""
        return self
        
    def set_text(self,text):
        """Set the text to be spell-checked.
        This method must be called, or the 'text' argument supplied
        to the constructor, before calling the 'next()' method.
        """
        # Convert to an array object if necessary
        if isinstance(text,basestring):
            if type(text) == unicode:
                self._text = array.array('u',text)
            else:
                self._text = array.array('c',text)
            self._use_tostring = True
        else:
            self._text = text
            self._use_tostring = False
        self._tokens = self._tokenize(self._text)
        
    def get_text(self):
        """Return the spell-checked text."""
        if self._use_tostring:
            return self._array_to_string(self._text)
        return self._text
        
    def _array_to_string(self,text):
        """Format an internal array as a standard string."""
        if text.typecode == 'u':
            return text.tounicode()
        return text.tostring()

    def wants_unicode(self):
        """Check whether the checker wants unicode strings.
        This method will return True if the checker wants unicode strings
        as input, False if it wants normal strings.  It's important to
	    provide the correct type of string to the checker.
	    """
        if self._text.typecode == 'u':
            return True
        return False

    def coerce_string(self,text,enc=None):
        """Coerce string into the required type.
        This method can be used to automatically ensure that strings
        are of the correct type required by this checker - either unicode
        or standard.  If there is a mis-match, conversion is done using
        python's default encoding unless another encoding is specified.
        """
        if self.wants_unicode():
            if not isinstance(text,unicode):
                if enc is None:
                    return text.decode()    
                else:
                    return text.decode(enc)
            return text
        if not isinstance(text,str):
            if enc is None:
                return text.encode()
            else:
                return text.encode(enc)
        return text
        
    def next(self):
        """Process text up to the next spelling error.
        
        This method is designed to support the iterator protocol.
        Each time it is called, it will advance the 'word' attribute
        to the next spelling error in the text.  When no more errors
        are found, it will raise StopIteration.
        
        The method will always return self, so that it can be used
        sensibly in common idoms such as:

            for err in checker:
                err.do_something()
        
        """
        # Find the next spelling error
        # The uncaught StopIteration from self._tokens.next()
        # will provide the StopIteration for this method
        while True:
            (word,pos) = self._tokens.next()
            # decode back to a regular string
            word = self._array_to_string(word)
            if self.dict.check(word):
                continue
            if word in self._ignore_words:
                continue
            self.word = word
            self.wordpos = pos
            if word in self._replace_words:
                self.replace(self._replace_words[word])
                continue
            break
        return self
    
    def replace(self,repl):
        """Replace the current erroneous word with the given string."""
        repl = self.coerce_string(repl)
        aRepl = array.array(self._text.typecode,repl)
        self.dict.store_replacement(self.word,repl)
        self._text[self.wordpos:self.wordpos+len(self.word)] = aRepl
        self._tokens.offset = self._tokens.offset + (len(repl)-len(self.word))
    
    def replace_always(self,word,repl=None):
        """Always replace given word with given replacement.
        If a single argumet is given, this is used to replace the
        current erroneous word.  If two arguments are given, that
        combination is added to the list for future use.
        """
        if repl is None:
            repl = word
            word = self.word
        repl = self.coerce_string(repl)
        word = self.coerce_string(word)
        self._replace_words[word] = repl
        if self.word == word:
            self.replace(repl)

    def ignore_always(self,word=None):
        """Add given word to list of words to ignore.
        If no word is given, the current erroneous word is added.
        """
        if word is None:
            word = self.word
        word = self.coerce_string(word)
        if word not in self._ignore_words:
            self._ignore_words.append(word)
            
    def add_to_personal(self,word=None):
        """Add given word to the personal word list.
        If no word is given, the current erroneous word is added.
        """
        if word is None:
            word = self.word
        self.dict.add_to_personal(word)
    
    def suggest(self,word=None):
        """Return suggested spellings for the given word.
        If no word is given, the current erroneous word is used.
        """
        if word is None:
            word = self.word
        suggs = self.dict.suggest(word)
        return suggs
    
    def check(self,word):
        """Check correctness of the given word."""
        return self.dict.check(word)

    def set_offset(self,off,whence=0):
        """Set the offset of the tokenisation routine.
        For more details on the purpose of the tokenisation offset,
        see the documentation of the 'enchant.tokenize' module.
        The optional argument whence indicates the method by
        which to change the offset:
            * 0 (the default) treats <off> as an increment
            * 1 treats <off> as a distance from the start
            * 2 treats <off> as a distance from the end
        """
        if whence == 0:
            self._tokens.offset = self._tokens.offset + off
        elif whence == 1:
            assert(off > 0)
            self._tokens.offset= off
        elif whence == 2:
            assert(off > 0)
            self._tokens.offset = len(self._text) - 1 - off
        else:
            raise ValueError("Invalid value for whence: %s"%(whence,))
    
    def leading_context(self,chars):
        """Get <chars> characters of leading context.
        This method returns up to <chars> characters of leading
        context - the text that occurs in the string immediately
        before the currently erroneous word.
        """
        start = max(self.wordpos - chars,0)
        context = self._text[start:self.wordpos]
        return self._array_to_string(context)
    
    def trailing_context(self,chars):
        """Get <chars> characters of trailing context.
        This method returns up to <chars> characters of trailing
        context - the text that occurs in the string immediately
        after the currently erroneous word.
        """
        start = self.wordpos + len(self.word)
        end = min(start + chars,len(self._text))
        context = self._text[start:end]
        return self._array_to_string(context)    
        
            
class TestChecker(unittest.TestCase):
    """TestCases for checking behavior of SpellChecker class."""
    
    def test_run1(self):
        """Test performance of a basic spellchecking run."""
        text = """This is sme text with a few speling erors in it. Its gret
        for checking wheather things are working proprly with the SpellChecker
        class. Not gret for much elss though."""
        chkr = SpellChecker("en_US",text=text)
        for n,err in enumerate(chkr):
            if n == 0:
                # Fix up "sme" -> "some" properly
                self.assertEqual(err.word,"sme")
                self.assertEqual(err.wordpos,8)
                self.assert_("some" in err.suggest())
                err.replace("some")
            if n == 1:
                # Ignore "speling"
                self.assertEqual(err.word,"speling")
            if n == 2:
                # Check context around "erors", and replace
                self.assertEqual(err.word,"erors")
                self.assertEqual(err.leading_context(5),"ling ")
                self.assertEqual(err.trailing_context(5)," in i")
                err.replace(u"errors")
            if n == 3:
                # Replace-all on gret as it appears twice
                self.assertEqual(err.word,"gret")
                err.replace_always("great")
            if n == 4:
                # First encounter with "wheather", move offset back
                self.assertEqual(err.word,"wheather")
                err.set_offset(-1*len(err.word))
            if n == 5:
                # Second encounter, fix up "wheather'
                self.assertEqual(err.word,"wheather")
                err.replace("whether")
            if n == 6:
                # Just replace "proprly", but also add an ignore
                # for "SpellChecker"
                self.assertEqual(err.word,"proprly")
                err.replace("properly")
                err.ignore_always("SpellChecker")
            if n == 7:
                # The second "gret" should have been replaced
                # So it's now on "elss"
                self.assertEqual(err.word,"elss")
                err.replace("else")
            if n > 7:
                self.fail("Extraneous spelling errors were found")
        text2 = """This is some text with a few speling errors in it. Its great
        for checking whether things are working properly with the SpellChecker
        class. Not great for much else though."""
        self.assertEqual(chkr.get_text(),text2)



#
#  This is the pyenchant setuptools script.
#  Originally developed by Ryan Kelly, 2004.
#
#  This script is placed in the public domain.
#

import ez_setup
ez_setup.use_setuptools()

from setuptools import setup, find_packages, Extension

import sys
import os
import shutil

# Location of the windows binaries, if available
WINDEPS = ".\\tools\\pyenchant-bdist-win32-sources\\build"

#  Cant seem to obtain version information from module, must hardcode
VERSION = "1.2.0"

# Package MetaData
NAME = "pyenchant"
DESCRIPTION = "Python bindings for the Enchant spellchecking system"
AUTHOR = "Ryan Kelly"
AUTHOR_EMAIL = "ryan@rfk.id.au"
URL = "http://pyenchant.sourceforge.net/"
LICENSE = "LGPL"
KEYWORDS = "spelling spellcheck enchant"


#  Module Lists
PACKAGES = find_packages()
EXT_MODULES = []
PKG_DATA = {}
EAGER_RES = []


#  The distutils/swig integration doesnt seem to cut it for this module
#  For now, enchant_wrap.c will need to be distributed as well.  At least
#  then people wont *have* to have swig installed.
#  Generate it using `swig -python -noproxy enchant.i` to ensure that
#  proxy class stubs are not generated.
swig_infile = os.path.join('enchant','enchant.i')
swig_outfile = os.path.join('enchant','enchant_wrap.c')
if not os.path.exists(swig_outfile) or \
       os.stat(swig_outfile).st_mtime < os.stat(swig_infile).st_mtime:
    print "Generating '%s'..." % (swig_outfile,)
    os.system('swig -python -noproxy -o %s %s' % (swig_outfile,swig_infile))


# Extension Objects
ext1 = Extension('enchant._enchant',['enchant/enchant_wrap.c'],
                 libraries=[],
                 library_dirs=[],
                )
 
#
# Build and distribution information is different on Windows
# The enchant library builds as 'enchant-1' instead of 'enchant'
#
# Also, there's the possibility of including pre-built support DLLs
# for the Windows installer.  They will be included if the directory
# <WINDEPS> exists when this script is run.  They are copied into
# the package directory so setuptools can locate them.
#
if sys.platform == "win32":
    # Copy local DLLs across if available
    if os.path.exists(WINDEPS):
      # Main DLLs
      libDir = os.path.join(WINDEPS,"lib")
      for fName in os.listdir(libDir):
        if fName[-3:] == "dll":
          shutil.copy(os.path.join(libDir,fName),".\\enchant\\")
      # Enchant plugins
      plugDir = os.path.join(WINDEPS,"lib\\enchant")
      for fName in os.listdir(plugDir):
        if fName[-3:] == "dll":
          shutil.copy(os.path.join(plugDir,fName),".\\enchant\\lib\\enchant\\")
      # Local Dictionaries
      dictPath = os.path.join(WINDEPS,"myspell")
      if os.path.isdir(dictPath):
        for dictName in os.listdir(dictPath):
          if dictName[-3:] in ["txt","dic","aff"]:
            shutil.copy(os.path.join(dictPath,dictName),
			".\\enchant\\share\\enchant\\myspell\\")
      dictPath = os.path.join(WINDEPS,"ispell")
      if os.path.isdir(dictPath):
        for dictName in os.listdir(dictPath):
          if dictName.endswith("hash") or dictName == "README.txt":
            shutil.copy(os.path.join(dictPath,dictName),
			".\\enchant\\share\\enchant\\ispell\\")
    # Set up additional compile info for C extension
    ext1.libraries.append("libenchant-1")
    ext1.library_dirs.append("enchant")
    PKG_DATA["enchant"] = ["*.dll", "lib/enchant/*.dll",
                           "share/enchant/myspell/*.*",
                           "share/enchant/ispell/*.*"]
    EAGER_RES = ["enchant/lib", "enchant/share"]
else:
    ext1.libraries.append("enchant")

EXT_MODULES.append(ext1)

##
##  Main call to setup() function
##

setup(name=NAME,
      version=VERSION,
      author=AUTHOR,
      author_email=AUTHOR_EMAIL,
      url=URL,
      description=DESCRIPTION,
      license=LICENSE,
      keywords=KEYWORDS,
      packages=PACKAGES,
      ext_modules=EXT_MODULES,
      package_data=PKG_DATA,
      eager_resources=EAGER_RES,
      test_suite="enchant.testsuite",
     )

